#!/bin/ksh
# ==============================================================================
# CPU Utilization Statistics plugin for Nagios
#
# Written by	:	Steve Bosek
# Release	: 	3.0 Michael Tosch
# Revision date : 	2016-03-11
# Description   : 	Nagios plugin (script) to check cpu utilization
#			statistics.
#		  	This script has been designed and written on Unix
#			plateform (Linux, Aix, Solaris, HP-UX),
#			requiring iostat or vmstat or sar as external program.
#			The script is used to query 4 of the key cpu statistics
#			(user,system,iowait,idle) at the same time.
#
# USAGE		: 	./check_cpu_stats.sh [-w <user,system,iowait>] [-c <user,system,iowait>] ( [ -i <intervals in second> ] [ -n <report number> ])
#
# Exemple: ./check_cpu_stats.sh
#	./check_cpu_stats.sh -w 75,55,30 -c 90,70,40
#	./check_cpu_stats.sh -w 75,55,30 -c 90,70,40 -i 3 -n 5
#
# ----------------------------------------------------------------------------------------
#
# TODO:  	- Support for MacOSX
#					- Search Binary Path (iostat,sar) or Add Path parameter when other as default
#

# Paths to commands used in this script.  These may have to be modified to match your system setup.
PATH=/bin:/usr/bin:/usr/sbin:/sbin
export PATH
set -f

# Nagios return codes
STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3

# Plugin parameters value if not define
LIST_WARNING_THRESHOLD=${LIST_WARNING_THRESHOLD:="75,55,30"}
LIST_CRITICAL_THRESHOLD=${LIST_CRITICAL_THRESHOLD:="90,70,40"}
INTERVAL_SEC=2
NUM_REPORT=2

print_usage() {
  PROGNAME=${0##*/}
  RELEASE="Revision 3.0"
  echo "
$PROGNAME $RELEASE - CPU Utilization check script for Nagios

Usage: check_cpu_stats.sh -w -c (-i -n)

  -w  Warning threshold in % for warn_user,warn_system,warn_iowait CPU (default : ${LIST_WARNING_THRESHOLD})
  Exit with WARNING status if cpu exceeds warn_n
  -c  Critical threshold in % for crit_user,crit_system,crit_iowait CPU (default : ${LIST_CRITICAL_THRESHOLD})
  Exit with CRITICAL status if cpu exceeds crit_n
  -i  Interval in seconds for iostat (default : ${INTERVAL_SEC})
  -n  Number report for iostat (default : ${NUM_REPORT})
  -h  Show this page

Usage: $PROGNAME
Usage: $PROGNAME --help
"
exit 0
}

print_help() {
  print_usage
  echo "
This plugin will check cpu utilization (user,system,Iowait,idle in %)
"
  exit 0
}

# Parse parameters
while [ $# -gt 0 ]; do
  case "$1" in
  -h | --help)
    print_help
    exit $STATE_OK
  ;;
  -v | --version)
    echo "$ELEASE"
    exit $STATE_OK
  ;;
  -w | --warning)
    shift
    LIST_WARNING_THRESHOLD=$1
  ;;
  -c | --critical)
    shift
    LIST_CRITICAL_THRESHOLD=$1
  ;;
  -i | --interval)
    shift
    INTERVAL_SEC=$1
  ;;
  -n | --number)
    shift
    NUM_REPORT=$1
  ;;
  *)  echo "Unknown argument: $1"
    print_usage
    exit $STATE_UNKNOWN
  ;;
  esac
  shift
done

# assume mksh
oIFS=$IFS; IFS=,
set -- $LIST_WARNING_THRESHOLD
if [ $# -ne 3 ]
then
  echo "ERROR : Bad count parameter in Warning Threshold"
  exit $STATE_WARNING
fi
USER_WARNING_THRESHOLD=$1 SYSTEM_WARNING_THRESHOLD=$2 IOWAIT_WARNING_THRESHOLD=$3
set -- $LIST_CRITICAL_THRESHOLD
if [ $# -ne 3 ]
then
  echo "ERROR : Bad count parameter in CRITICAL Threshold"
  exit $STATE_WARNING
fi
USER_CRITICAL_THRESHOLD=$1 SYSTEM_CRITICAL_THRESHOLD=$2 IOWAIT_CRITICAL_THRESHOLD=$3
IFS=$oIFS

if [ ${USER_WARNING_THRESHOLD} -ge ${USER_CRITICAL_THRESHOLD} ] ||
  [ ${SYSTEM_WARNING_THRESHOLD} -ge ${SYSTEM_CRITICAL_THRESHOLD} ] ||
  [ ${IOWAIT_WARNING_THRESHOLD} -ge ${IOWAIT_CRITICAL_THRESHOLD} ]
then
  echo "ERROR : Critical CPU Threshold lower as Warning CPU Threshold "
  exit $STATE_WARNING
fi

# CPU Utilization Statistics Unix Plateform ( Linux,AIX,Solaris,HP-UX are supported )
case `uname` in
Linux )
# assume mksh
# set -- `iostat -c $INTERVAL_SEC $NUM_REPORT | sed -n 's/[,.][0-9][0-9]*//g; /./h; ${x;p;}'`
# USER=$1 "SYSTEM=$3 IOWAIT=$4 IDLE=${6-$5}
  set -- `vmstat $INTERVAL_SEC $NUM_REPORT | tail -1`
  USER=${13} SYSTEM=${14} IOWAIT=${16} IDLE=${15}
;;
AIX )
  set -- `iostat -t $INTERVAL_SEC $NUM_REPORT | sed -n 's/,/./g; $p'`
  USER=$3 SYSTEM=$4 IDLE=$5 IOWAIT=$6
;;
SunOS )
  set -- `iostat -c $INTERVAL_SEC $NUM_REPORT | tail -1`
  USER=$1 SYSTEM=$2 IOWAIT=$3 IDLE=$4
;;
HP-UX)
  set -- `sar $INTERVAL_SEC $((NUM_REPORT - 1)) | tail -1`
  USER=$2 SYSTEM=$3 IOWAIT=$4 IDLE=$5
;;
*)
  echo "UNKNOWN: $OS not yet supported by this plugin."
  exit $STATE_UNKNOWN
;;
esac

NAGIOS_DATA="user=${USER}% system=${SYSTEM}%, iowait=${IOWAIT}%, idle=${IDLE}% | CpuUser=${USER}%;${USER_WARNING_THRESHOLD};${USER_CRITICAL_THRESHOLD};0; CpuSystem=${SYSTEM}%;${SYSTEM_WARNING_THRESHOLD};${SYSTEM_CRITICAL_THRESHOLD};0; CpuIowait=${IOWAIT}%;${IOWAIT_WARNING_THRESHOLD};${IOWAIT_CRITICAL_THRESHOLD};0; CpuIdle=${IDLE}%"


# Return
if [ ${USER} -ge $USER_CRITICAL_THRESHOLD ] ||
  [ ${SYSTEM} -ge $SYSTEM_CRITICAL_THRESHOLD ] ||
  [ ${IOWAIT} -ge $IOWAIT_CRITICAL_THRESHOLD ]
then
  echo "CPU STATS CRITICAL : ${NAGIOS_DATA}"
  exit $STATE_CRITICAL
fi
if [ ${USER} -ge $USER_WARNING_THRESHOLD ] ||
  [ ${SYSTEM} -ge $SYSTEM_WARNING_THRESHOLD ] ||
  [ ${IOWAIT} -ge $IOWAIT_WARNING_THRESHOLD ]
then
  echo "CPU STATS WARNING : ${NAGIOS_DATA}"
  exit $STATE_WARNING
fi
echo "CPU STATS OK : ${NAGIOS_DATA}"
exit $STATE_OK
